package com.gxzc.zen.api.util

import com.github.tobato.fastdfs.domain.FileInfo
import com.github.tobato.fastdfs.domain.StorePath
import com.github.tobato.fastdfs.exception.FdfsServerException
import com.github.tobato.fastdfs.service.AppendFileStorageClient
import com.github.tobato.fastdfs.service.FastFileStorageClient
import com.gxzc.zen.api.sys.model.SysUploadInfo
import com.gxzc.zen.common.util.FileUtil
import com.gxzc.zen.common.util.SpringContextHolder
import com.gxzc.zen.common.util.upload.FdfsFile
import com.gxzc.zen.rpc.api.sys.RUploadService
import org.apache.commons.io.FilenameUtils
import org.slf4j.LoggerFactory
import java.io.File

/**
 * FastDFS 工具类 <br>
 * 后端上传文件到 分布式文件服务器
 * @author NorthLan
 * @date 2018/9/1
 * @url https://noahlan.com
 */
object FileSystemUtil {
    private val logger = LoggerFactory.getLogger(FileSystemUtil::class.java)

    private val appendFileStorageClient: AppendFileStorageClient by lazy { SpringContextHolder.getBean(AppendFileStorageClient::class.java)!! }
    private val fastFileStorageClient: FastFileStorageClient by lazy { SpringContextHolder.getBean(FastFileStorageClient::class.java)!! }
    private val rUploadService: RUploadService by lazy { SpringContextHolder.getBean(RUploadService::class.java)!! }

    /**
     * 上传文件 <br>
     * 若文件已存在则不会再次上传,且返回新的路径(软链接)
     * @param path 文件路径
     * @param groupName 上传到指定组,默认自动分配
     */
    fun uploadFile(path: String, groupName: String? = null): StorePath {
        return uploadFile(File(path), groupName)
    }

    /**
     * 上传文件 <br>
     * 若文件已存在则不会再次上传,且返回新的路径(软链接)
     * @param path 文件路径
     * @param groupName 上传到指定组,默认自动分配
     */
    fun uploadFile(file: File, groupName: String? = null): StorePath {
        file.inputStream().use {
            return appendFileStorageClient.uploadAppenderFile(groupName, it, file.length(), FilenameUtils.getExtension(file.name))
        }
    }

    /**
     * 上传文件并入库 <br>
     * 若文件已存在则不会再次上传,且返回原有信息
     * @param path 文件路径
     * @param groupName 上传到指定组,默认自动分配
     * @return 数据信息
     */
    fun uploadFileDB(path: String, groupName: String? = null): SysUploadInfo? {
        return uploadFileDB(File(path), groupName)
    }

    /**
     * 上传文件并入库 <br>
     * 若文件已存在则不会再次上传,且返回原有信息
     * @param path 文件路径
     * @param groupName 上传到指定组,默认自动分配
     * @return 数据信息
     */
    fun uploadFileDB(file: File, groupName: String? = null): SysUploadInfo? {
        val md5 = FileUtil.md5HeadTail(file)
        val fileInfo = rUploadService.getUploadInfoByMd5(md5)
        if (fileInfo != null) {
            return fileInfo
        }
        val storePath = uploadFile(file, groupName)
        return rUploadService.addFileInfo(file.name, md5, file.length(), file.lastModified(), file.path, FdfsFile(storePath.group, storePath.path))
    }

    /**
     * 查询文件信息
     * @param groupName 组名
     * @param path 名
     * @return 若文件服务器上无此文件,则返回null
     */
    fun queryFileInfo(groupName: String, path: String): FileInfo? {
        return try {
            fastFileStorageClient.queryFileInfo(groupName, path)
        } catch (e: FdfsServerException) {
            null
        }
    }

    /**
     * 下载并保存到指定位置
     * @param groupName 组名
     * @param path path
     * @param outputPath 指定位置(包括文件名,建议使用绝对路径)
     */
    fun downloadFile(groupName: String, path: String, outputPath: String): File {
        return fastFileStorageClient.downloadFile(groupName, path) {
            FileUtil.writeFile(it, outputPath)
        }
    }

    /**
     * 删除文件
     * 删除失败抛出异常
     */
    fun deleteFile(groupName: String, path: String) {
        fastFileStorageClient.deleteFile(groupName, path)
    }
}